/*
 *  Main game logic
 *
 *
 */


#include <unistd.h>
#include <stdlib.h>

#include "debug.h"

#include "errors.h"
#include "protocol.h"
#include "network.h"
#include "map.h"
#include "packages.h"
#include "logic.h"
#include "game.h"


void game_debugcommand(ll_rcvlist *cmds);



int game_main(char *host, int port)
{
	int rval;
	mapdata currentmap;
	init_stats mystats;
	ll_rcvlist *commands;
	ll_packagelist *packages;
	sendcommand execute;



	/* Connect to the server */
	rval=net_connect(host,port);

	if (FAILURE(rval))
	{
		DEBUG_PRINT("Could not connect to server.\n");
		return rval;
	}


	/* Send hello */
	rval=proto_sendhello();

	if (FAILURE(rval))
	{
		DEBUG_PRINT("Failed on hello send.\n");
		return rval;
	}
	/* Receive board */
	rval=proto_receiveboard(&currentmap);

	if (FAILURE(rval))
	{
		DEBUG_PRINT("Failed on map receive.\n");
		return rval;
	}

	rval=map_PrintMap(&currentmap);
	if (FAILURE(rval))
	{
		DEBUG_PRINT("Failed on map print.\n");
		return rval;
	}

	rval=map_InitializeMap(&currentmap);
	if (FAILURE(rval))
	{
		DEBUG_PRINT("Failed on map init.\n");
		return rval;
	}



	/* Receive player configuration */
	rval=proto_receivestats(&mystats);
	if (FAILURE(rval))
	{
		DEBUG_PRINT("Failed on player stats init.\n");
		return rval;
	}

	ginfo.myuid=mystats.playernum;
	ginfo.my_weightmax=mystats.maxweight;
	ginfo.my_bidleft=mystats.bidleft;
	ginfo.turn=1;
	ginfo.my_score=0;



	commands=NULL;

	/* Receive initial server response */
	rval=proto_receivecommands(&commands);
	if (FAILURE(rval))
	{
		DEBUG_PRINT("Failed on receive commands.\n");
		return rval;
	}


	game_debugcommand(commands);
	logic_parsecommands(commands);

	ginfo.turn++;

#ifdef ANSI_VIEW
	DEBUG_PRINT("\e[2J");
#endif

	while(1)
	{
		/* Get list of packages here */
		rval=proto_receivepackages(&packages);
		if (FAILURE(rval))
		{
			DEBUG_PRINT("Failed on receive packages.\n");
			return rval;
		}
		
		packages_total_sawlist(packages,ginfo.my_x,ginfo.my_y);



		/* Decide what to do */

		rval=logic_decision(&execute);
		if (FAILURE(rval))
		{
			DEBUG_PRINT("Failed on logic decision.\n");
			return rval;
		}

		/* Decrease the bid */
		ginfo.my_bidleft-=execute.bid;

		/* Send the command */
		rval=proto_sendcommand(&execute);
		if (FAILURE(rval))
		{
			DEBUG_PRINT("Failed on send command.\n");
			return rval;
		}

		

		/* See what happened */

		rval=proto_receivecommands(&commands);
		if (FAILURE(rval))
		{
			DEBUG_PRINT("Failed on receive commands.\n");
			return rval;
		}


		game_debugcommand(commands);
		logic_parsecommands(commands);

		if (ginfo.my_bidleft<=0)
		{
#ifdef ANSI_VIEW
			DEBUG_PRINT("\e[2J");
#endif
			/* Print stats */
			
			return 0;
		}



#ifdef ANSI_VIEW
		map_PrintAnsiMap();
#endif

		ginfo.turn++;

	}

	



	/* Close the socket and exit */
	rval=net_close();
	if (FAILURE(rval))
	{
		DEBUG_PRINT("Failed to close socket.\n");
		return rval;
	}

	return 0;
}





void game_debugcommand(ll_rcvlist *cmds)
{
#ifdef DEBUG
#ifdef DEBUG_LOGIC
	static char outbuffer[1024];
	ll_oplist *cur_op;

	while (cmds!=NULL)
	{
		sprintf(outbuffer,"Player %d:",cmds->playernum);
		DEBUG_PRINT(outbuffer);

		switch (cmds->command)
		{
		case CMD_RCV_MOVE:
			switch (cmds->op1)
			{
				case DIRECTION_N:
					sprintf(outbuffer,"MOVEMENT NORTH\n");
					break;
					
				case DIRECTION_S:
					sprintf(outbuffer,"MOVEMENT SOUTH\n");
					break;
				case DIRECTION_E:
					sprintf(outbuffer,"MOVEMENT EAST\n");
					break;
				case DIRECTION_W:
					sprintf(outbuffer,"MOVEMENT WEST\n");
					break;
				default:
					sprintf(outbuffer,"UNKNOWN MOVEMENT VALUE %d\n",cmds->op1);
					break;
			}
			break;
		case CMD_RCV_POS:
			sprintf(outbuffer,"Position %d, %d.\n",cmds->op1,cmds->op2);
			break;
		case CMD_RCV_PICK:
			sprintf(outbuffer,"Picked up package %d.\n",cmds->op1);
			break;
		case CMD_RCV_DROP:
			sprintf(outbuffer,"Dropped package %d.\n",cmds->op1);
			break;
		default:
			sprintf(outbuffer,"Unknown command code %d. OP1: %d OP2: %d\n",cmds->command,cmds->op1,cmds->op2);
		break;
		}

		DEBUG_PRINT(outbuffer);

		cmds=cmds->next;
	}

	

#endif
#endif

}
